from pathlib import Path
import plotly.graph_objs as go
from django.shortcuts import render
from plotly.offline import plot
from plotly.graph_objs import Histogram
from select import select
import pandas as pd
import os
from django.conf import settings
from django.contrib.auth.decorators import login_required, permission_required
from datetime import datetime, timedelta
import plotly.express as px
from scipy.stats import t
import numpy as np

from file_manager .models import DataAnalysisQueue, SampleRecord, \
    SavedVisualization, VisualizationApp, UserSettings

APPFOLDER = "media/primary_storage/systemfiles/QC_Data_Comparison/data/"
# for download link
url_base = "primary_storage/systemfiles/QC_Data_Comparison/data/"

# app name, must be the same as in the database
APPNAME = "QC Data Comparison"
Group_LIST_NAME = ["group1_record", "group2_record",
                   "group3_record", "group4_record",
                   "group5_record", "group6_record"]
Group_NAME = ["group1_name", "group2_name", "group3_name",
              "group4_name", "group5_name", "group6_name"]

# for pop up warning message


@ login_required
def view(request):
    plot_div = None
    saved_settings = None
    message = ""

    if request.method == 'POST':
        selected_visualization = SavedVisualization.objects.filter(
            pk=request.POST.get("Select_exist_analysis")).first()
        settings = {
            "group1_name": request.POST.get('group1_name'),
            "group2_name": request.POST.get('group2_name'),
            "group3_name": request.POST.get('group3_name'),
            "group4_name": request.POST.get('group4_name'),
            "group5_name": request.POST.get('group5_name'),
            "group6_name": request.POST.get('group6_name'),
            "group1_record": request.POST.getlist('group1_record'),
            "group2_record": request.POST.getlist('group2_record'),
            "group3_record": request.POST.getlist('group3_record'),
            "group4_record": request.POST.getlist('group4_record'),
            "group5_record": request.POST.getlist('group5_record'),
            "group6_record": request.POST.getlist('group6_record'),
            "zero_value_option": request.POST.get('zero_value_option'),
            "plot_qc_value_type": request.POST.get('plot_qc_value_type'),
            "plot_errorbar_type": request.POST.get('plot_errorbar_type'),
            "plot_label_type": request.POST.get('plot_label_type'),


        }

        for item in Group_LIST_NAME:
            if settings[item] != []:
                settings[item] = \
                    list(map(lambda x: {
                        "value": x, "option": SampleRecord.objects.filter(
                            pk=x).first().record_name}, settings[item]))
        if 'start_new_analysis' in request.POST:
            # create new SavedVisualization object:
            formdata = {
                "visualization_app":
                    VisualizationApp.objects.filter(name=APPNAME).first(),
                "visual_creator": request.user,
                "visualization_name": request.POST.get('new_analysis_name'),
                "settings": settings
            }
            SavedVisualization.objects.create(**formdata)
        elif 'delete_saved_visualization' in request.POST:
            # delete SavedVisualization :
            if request.user == selected_visualization.visual_creator:
                SavedVisualization.objects.filter(
                    pk=selected_visualization.pk).delete()
            else:
                message = "You are not allowed to delete this analysis!"
        elif 'load_saved_visualization' in request.POST:
            # load SavedVisualization :
            saved_settings = selected_visualization.settings
            saved_settings["new_analysis_name"] = \
                selected_visualization.visualization_name
        elif 'save_plot' in request.POST and \
                request.POST.get('new_analysis_name') != "":

            if request.user == selected_visualization.visual_creator:
                selected_visualization.visualization_name = request.POST.get(
                    'new_analysis_name')
                selected_visualization.settings = settings
                selected_visualization.save()
                saved_settings = selected_visualization.settings
                saved_settings["new_analysis_name"] = \
                    selected_visualization.visualization_name

                plot_div = plot_ID_Histogram(selected_visualization)
            else:
                message = "You are not allowed to modifiy " \
                    "or plot this analysis!"
        else:
            message = "Something went wrong! did you left analysis name empty?"
    current_saved_visualization = SavedVisualization.objects.filter(
        visualization_app=VisualizationApp.objects.filter(
            name=APPNAME).first()).order_by('-pk')
    args = {
        "plot_div": plot_div,
        "SampleRecord":
            SampleRecord.objects.all().order_by('-pk'),
        "SavedVisualization": current_saved_visualization,
        "formdata": request.POST,
        "saved_settings": saved_settings,
        "message": message,

    }

    return render(request,
                  'filemanager/QC_data_comparison.html', args)


def plot_ID_Histogram(selected_visualization):
    df = pd.DataFrame()
    username = selected_visualization.visual_creator.username
    # get plot value type according to user setting
    # (e.g., protein, peptide, etc.)
    plot_value_type_str = None
    plot_value_type = selected_visualization.settings["plot_qc_value_type"]

    current_user_settings = UserSettings.objects.filter(
        user=selected_visualization.visual_creator).first()
    plot_value_type_str = getattr(
        current_user_settings, "qc_"+plot_value_type+"_name")

    for n in range(len(Group_LIST_NAME)):
        condition_name = selected_visualization.settings[Group_NAME[n]]
        if selected_visualization.settings[Group_LIST_NAME[n]] != []:
            for run in selected_visualization.settings[Group_LIST_NAME[n]]:
                QC_run = SampleRecord.objects.filter(
                    pk=run["value"]).first().quanlity_check
                plot_value = getattr(
                    QC_run, "output_QC_number_"+plot_value_type)
                if selected_visualization.settings["zero_value_option"] == "false" and plot_value == 0:
                    continue  # skip zero value
                df = pd.concat([df, pd.DataFrame(
                    {"sample": [condition_name], "value": [plot_value]})], ignore_index=True)
    df = df.groupby(["sample"]).agg({
        'value': ['mean', 'std', 'count']})
    df.columns = ['mean', 'stdev', 'n']

    df = df.reset_index()
    df["confInt"] = t.ppf(0.975, df['n']-1) * df['stdev']/np.sqrt(df['n'])
    if selected_visualization.settings["plot_errorbar_type"] == "1":
        error_bar = "stdev"
    elif selected_visualization.settings["plot_errorbar_type"] == "2":
        error_bar = "confInt"
    else:
        error_bar = None

    if (selected_visualization.settings["plot_label_type"] == "2"):
        total_labels = [{"x": x, "y": total*1.15, "text": str(
            int(total)), "showarrow": False} for x, total in zip(df["sample"], df["mean"])]
    else:
        total_labels = []

    img_dir = os.path.join(APPFOLDER, "images/")
    if not os.path.exists(img_dir):
        Path(img_dir).mkdir(parents=True)

    download_link = f"<a href='/files/{url_base}/images/" \
        f"{username}_QC_Data_Plot.svg' "\
        f"style='opacity:10'>SVG \u21E9 </a>"
    svg_link_dict = {
        "text": download_link,
        "font": dict(size=15),
        "x": 1.2,
        "y": -0.1,
        "showarrow": False,
        "xref": "paper",
        "yref": "paper",
        "xanchor": "left",
        "yanchor": "bottom"

    }
    total_labels.append(svg_link_dict)

    # generate the csv data for download
    data_dir = os.path.join(APPFOLDER, "csv/")
    if not os.path.exists(data_dir):
        Path(data_dir).mkdir(parents=True)
    df.to_csv(os.path.join(
        data_dir, f"{username}_QC_data_comparison.csv"), index=False)
    data_download_link = f"<a href='/files/{url_base}/csv/" \
        f"{username}_QC_data_comparison.csv' "\
        f"style='opacity:10'>CSV \u21E9 </a>"

    data_link_dict = {
        "text": data_download_link,
        "font": dict(size=15),
        "x": 1.2,
        "y": -0.15,
        "showarrow": False,
        "xref": "paper",
        "yref": "paper",
        "xanchor": "left",
        "yanchor": "bottom"

    }
    total_labels.append(data_link_dict)
    fig = px.bar(df,
                 x='sample',
                 y='mean',
                 color='sample',
                 error_y=error_bar,
                 width=980,
                 height=630,
                 )

    fig.update_layout(
        title=plot_value_type_str + " Identification Summary",
        xaxis_title='Experiments',
        yaxis_title='IDs',
        annotations=total_labels,
        font=dict(
            family="Arial Black",
            size=22,
            color="Black"
        ))
    fig.write_image(os.path.join(
        img_dir, f"{username}_QC_Data_Plot.svg"))

    plot_div = plot(fig, output_type="div")

    return plot_div
